@@ -1,5 +1,5 @@
 // Copyright (C) 2015-2016 Bart Smeets and Iñaki Ucar
-// Copyright (C) 2016-2019 Iñaki Ucar
+// Copyright (C) 2016-2020 Iñaki Ucar
 //
 // This file is part of simmer.
 //
@@ -49,6 +49,7 @@ namespace simmer {
     typedef UMAP<std::string, ArrTime> ResTime;
     typedef UMAP<int, Resource*> SelMap;
     typedef VEC<Resource*> ResVec;
+    typedef VEC<Activity*> ActVec;
 
     CLONEABLE(Arrival)
 
@@ -64,14 +65,14 @@ namespace simmer {
     */
     Arrival(Simulator* sim, const std::string& name, int mon, Order order,
             Activity* first_activity, int priority = 0)
-      : Process(sim, name, mon, priority), order(order), paused(false),
+      : Process(sim, name, mon, priority), order(order), paused(0),
         clones(new int(0)), activity(first_activity), timer(NULL),
-        dropout(NULL), batch(NULL) { init(); }
+        dropout(NULL), batch(NULL), act_shd(new ActVec()) { init(); }
 
     Arrival(const Arrival& o)
       : Process(o), order(o.order), paused(o.paused), clones(o.clones),
         activity(NULL), attributes(o.attributes), timer(NULL),
-        dropout(NULL), batch(NULL) { init(); }
+        dropout(NULL), batch(NULL), act_shd(o.act_shd) { init(); }
 
     ~Arrival() { reset(); }
 
@@ -108,23 +109,25 @@ namespace simmer {
     }
 
     void restart() {
+      if (--paused) return;
       set_busy(sim->now() + status.remaining);
       activate(status.remaining);
       set_remaining(0);
-      paused = false;
     }
 
     void pause() {
+      if (paused++) return;
       deactivate();
+      if (status.busy_until < sim->now())
+        return;
       unset_busy(sim->now());
       if (status.remaining && order.get_restart()) {
         unset_remaining();
         activity = activity->get_prev();
       }
-      paused = true;
     }
 
-    bool is_paused() const { return paused; }
+    bool is_paused() const { return paused > 0; }
 
     void stop() {
       deactivate();
@@ -179,20 +182,32 @@ namespace simmer {
       if (!ptr) Rcpp::stop("illegal register of arrival '%s'", name); // # nocov
       batch = ptr;
     }
-
     void unregister_entity(Batched* ptr) {
       if (ptr != batch)
         Rcpp::stop("illegal unregister of arrival '%s'", name); // # nocov
       batch = NULL;
     }
 
+    void register_entity(Activity* ptr, bool shared=false) {
+      if (!ptr) Rcpp::stop("illegal register of arrival '%s'", name); // # nocov
+      ActVec& act = shared ? *act_shd : act_this;
+      act.push_back(ptr);
+    }
+    void unregister_entity(Activity* ptr, bool shared=false) {
+      ActVec& act = shared ? *act_shd : act_this;
+      ActVec::iterator search = std::find(act.begin(), act.end(), ptr);
+      if (!ptr || search == act.end())
+        Rcpp::stop("illegal unregister of arrival '%s'", name); // # nocov
+      act.erase(search);
+    }
+
     void set_dropout(Activity* next) { dropout = next; }
 
     void set_renege(double timeout, Activity* next, bool keep_seized) {
       cancel_renege();
       timer = new Task(sim, "Renege-Timer",
                        BIND(&Arrival::renege, this, next, keep_seized),
-                       PRIORITY_MIN);
+                       PRIORITY_MAX);
       timer->activate(timeout);
     }
 
@@ -214,7 +229,7 @@ namespace simmer {
     }
 
   private:
-    bool paused;
+    int paused;
     int* clones;          /**< number of active clones */
     ArrStatus status;     /**< arrival timing status */
     ArrTime lifetime;     /**< time spent in the whole trajectory */
@@ -227,6 +242,8 @@ namespace simmer {
     Activity* dropout;    /**< drop-out trajectory */
     Batched* batch;       /**< batch that contains this arrival */
     ResVec resources;     /**< resources that contain this arrival */
+    ActVec act_this;      /**< activities that contain this arrival */
+    ActVec* act_shd;      /**< activities that contain any clone */
 
     void init() {
       (*clones)++;
@@ -235,8 +252,14 @@ namespace simmer {
 
     void reset() {
       cancel_renege();
-      if (!--(*clones))
+      foreach_ (ActVec::value_type& itr, act_this)
+        itr->remove(this);
+      if (!--(*clones)) {
+        foreach_ (ActVec::value_type& itr, *act_shd)
+          itr->remove(this);
+        delete act_shd;
         delete clones;
+      }
       sim->unregister_arrival(this);
     }
 
@@ -251,7 +274,7 @@ namespace simmer {
       sim->mon->record_release(name, start, sim->now(), activity, resource);
     }
 
-    void leave_resources(bool was_batched, bool keep_seized);
+    void leave_resources(bool keep_seized);
 
     virtual void update_activity(double value) {
       lifetime.activity += value;
